/*
* The contents of this file are subject to the terms
* of the Common Development and Distribution License
* (the License).  You may not use this file except in
* compliance with the License.
*
* You can obtain a copy of the license at
* https://glassfish.dev.java.net/public/CDDLv1.0.html or
* glassfish/bootstrap/legal/CDDLv1.0.txt.
* See the License for the specific language governing
* permissions and limitations under the License.
*
* When distributing Covered Code, include this CDDL
* Header Notice in each file and include the License file
* at glassfish/bootstrap/legal/CDDLv1.0.txt.
* If applicable, add the following below the CDDL Header,
* with the fields enclosed by brackets [] replaced by
* you own identifying information:
* "Portions Copyrighted [year] [name of copyright owner]"
*
* Copyright 2006 Sun Microsystems, Inc. All rights reserved.
*
*/

package org.glassfish.scripting.rails;

import org.glassfish.api.admin.ParameterNames;
import org.glassfish.api.admin.ServerEnvironment;
import org.glassfish.api.container.EndpointRegistrationException;
import org.glassfish.api.container.RequestDispatcher;
import org.glassfish.api.deployment.Deployer;
import org.glassfish.api.deployment.DeploymentContext;
import org.glassfish.api.deployment.MetaData;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Service;

import java.util.logging.Level;

/**
 * Deployer for Rails applications
 *
 * @author Jerome Dochez
 * @author Vivek Pandey
 */
@Service
public class RailsDeployer implements Deployer<RailsContainer, RailsApplication> {

    @Inject
    ServerEnvironment env;

    @Inject
    RequestDispatcher dispatcher;

    public boolean prepare(DeploymentContext context) {
        return true;
    }

    /**
     * Loads the meta date associated with the application.
     *
     * @parameters type type of metadata that this deployer has declared providing.
     */
    public <T> T loadMetaData(Class<T> type, DeploymentContext context) {
        return null;
    }

    /**
     * Returns the meta data assocated with this Deployer
     *
     * @return the meta data for this Deployer
     */
    public MetaData getMetaData() {
        return new MetaData(false, null, null, null);
    }

    public RailsApplication load(RailsContainer container, DeploymentContext context) {
        String contextRoot = getContextRoot(context);
        String railsRoot = context.getSource().getURI().getSchemeSpecificPart();
        RailsApplication adapter = new RailsApplication(env, railsRoot, context.getLogger(), contextRoot);

        context.getLogger().info("Loading application " +
                context.getCommandParameters().getProperty(ParameterNames.NAME)
                + " at " + contextRoot);

        //@TODO change EndportRegistrationException processing if required
        try {
            dispatcher.registerEndpoint(contextRoot, null, adapter, adapter);
        } catch (EndpointRegistrationException e) {
            context.getLogger().log(Level.WARNING, "Error registering RailsAdapter", e);
        }
        return adapter;
    }

    public void unload(RailsApplication container, DeploymentContext context) {
        try {
            dispatcher.unregisterEndpoint(container.getContextRoot());
        } catch (EndpointRegistrationException e) {
            context.getLogger().log(Level.SEVERE, "Exception while unloading ruby application", e);
        }
    }

    public void clean(DeploymentContext context) {
    }

    private String getContextRoot(DeploymentContext context) {
        String contextRoot = context.getCommandParameters().getProperty(ParameterNames.CONTEXT_ROOT);
        if (contextRoot == null) {
            contextRoot = env.getStartupContext().getArguments().get("--contextroot");
        }
        if (contextRoot == null || contextRoot.length() == 0) {
            contextRoot = "/" + context.getCommandParameters().getProperty(ParameterNames.NAME);
        }
        return contextRoot;
    }
}

